@@ -99,6 +99,16 @@ namespace Rstats {
     };
   }
 
+  typedef SV* Character;
+  typedef std::complex<NV> Complex;
+  typedef NV Double;
+  typedef IV Integer;
+
+  typedef std::vector<Rstats::Character> CharacterVector;
+  typedef std::vector<Rstats::Complex> ComplexVector;
+  typedef std::vector<Rstats::Double> DoubleVector;
+  typedef std::vector<Rstats::Integer> IntegerVector;
+  
   namespace Util {
     IV is_perl_number(SV*);
     SV* cross_product(SV*);
@@ -302,17 +312,11 @@ namespace Rstats {
     IV is_nan(IV);
   }
 
-  struct Vector {
+  typedef struct {
     Rstats::Type::Enum type;
     std::map<IV, IV>* na_positions;
     void* values;
-  };
-  
-  template <typename T>
-  struct Vector2 {
-    std::vector<T>* values;
-    std::map<IV, IV>* na_positions;
-  };
+  } Vector;
 
   // Macro for Rstats::Vector
 # define RSTATS_DEF_VECTOR_FUNC_UN_IS(FUNC_NAME, ELEMENT_FUNC_NAME) \
@@ -323,23 +327,23 @@ namespace Rstats {
       switch (type) { \
         case Rstats::Type::CHARACTER : \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_integer_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_character_value(v1, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Character>(v1, i))); \
           } \
           break; \
         case Rstats::Type::COMPLEX : \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_integer_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_complex_value(v1, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i))); \
           } \
           break; \
         case Rstats::Type::DOUBLE : \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_integer_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_double_value(v1, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Double>(v1, i))); \
           } \
           break; \
         case Rstats::Type::INTEGER : \
         case Rstats::Type::LOGICAL : \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_integer_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_integer_value(v1, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i))); \
           } \
           break; \
         default: \
@@ -348,7 +352,7 @@ namespace Rstats {
       } \
       for (IV i = 0; i < length; i++) { \
         if (Rstats::VectorFunc::exists_na_position(v1, i)) { \
-          Rstats::VectorFunc::set_integer_value(v2, i, 0); \
+          Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, 0); \
         } \
       } \
       return v2; \
@@ -363,20 +367,20 @@ namespace Rstats {
         case Rstats::Type::COMPLEX : \
           v2 = Rstats::VectorFunc::new_complex(length); \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_complex_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_complex_value(v1, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Complex>(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i))); \
           } \
           break; \
         case Rstats::Type::DOUBLE : \
           v2 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_double_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_double_value(v1, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Double>(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Double>(v1, i))); \
           } \
           break; \
         case Rstats::Type::INTEGER : \
         case Rstats::Type::LOGICAL : \
           v2 = Rstats::VectorFunc::new_integer(length); \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_integer_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_integer_value(v1, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i))); \
           } \
           break; \
         default: \
@@ -396,20 +400,20 @@ namespace Rstats {
         case Rstats::Type::COMPLEX : \
           v2 = Rstats::VectorFunc::new_complex(length); \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_complex_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_complex_value(v1, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Complex>(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i))); \
           } \
           break; \
         case Rstats::Type::DOUBLE : \
           v2 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_double_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_double_value(v1, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Double>(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Double>(v1, i))); \
           } \
           break; \
         case Rstats::Type::INTEGER : \
         case Rstats::Type::LOGICAL : \
           v2 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_double_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_integer_value(v1, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Double>(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i))); \
           } \
           break; \
         default: \
@@ -429,20 +433,20 @@ namespace Rstats {
         case Rstats::Type::COMPLEX : \
           v2 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_double_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_complex_value(v1, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Double>(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i))); \
           } \
           break; \
         case Rstats::Type::DOUBLE : \
           v2 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_double_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_double_value(v1, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Double>(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Double>(v1, i))); \
           } \
           break; \
         case Rstats::Type::INTEGER : \
         case Rstats::Type::LOGICAL : \
           v2 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_double_value(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_integer_value(v1, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Double>(v2, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i))); \
           } \
           break; \
         default: \
@@ -467,18 +471,18 @@ namespace Rstats {
       switch (type) { \
         case Rstats::Type::CHARACTER : \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_integer_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_character_value(v1, i), Rstats::VectorFunc::get_character_value(v2, i)) ? 1 : 0); \
+            Rstats::VectorFunc::set_value<Rstats::Integer>(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Character>(v1, i), Rstats::VectorFunc::get_value<Rstats::Character>(v2, i)) ? 1 : 0); \
           } \
           break; \
         case Rstats::Type::COMPLEX : \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_integer_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_complex_value(v1, i), Rstats::VectorFunc::get_complex_value(v2, i)) ? 1 : 0); \
+            Rstats::VectorFunc::set_value<Rstats::Integer>(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i), Rstats::VectorFunc::get_value<Rstats::Complex>(v2, i)) ? 1 : 0); \
           } \
           break; \
         case Rstats::Type::DOUBLE : \
           for (IV i = 0; i < length; i++) { \
             try {\
-              Rstats::VectorFunc::set_integer_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_double_value(v1, i), Rstats::VectorFunc::get_double_value(v2, i)) ? 1 : 0); \
+              Rstats::VectorFunc::set_value<Rstats::Integer>(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Double>(v1, i), Rstats::VectorFunc::get_value<Rstats::Double>(v2, i)) ? 1 : 0); \
             } catch (const char* e) {\
               Rstats::VectorFunc::add_na_position(v3, i);\
             }\
@@ -488,7 +492,7 @@ namespace Rstats {
         case Rstats::Type::LOGICAL : \
           for (IV i = 0; i < length; i++) { \
             try {\
-              Rstats::VectorFunc::set_integer_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_integer_value(v1, i), Rstats::VectorFunc::get_integer_value(v2, i)) ? 1 : 0); \
+              Rstats::VectorFunc::set_value<Rstats::Integer>(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i), Rstats::VectorFunc::get_value<Rstats::Integer>(v2, i)) ? 1 : 0); \
             }\
             catch (const char* e) {\
               Rstats::VectorFunc::add_na_position(v3, i);\
@@ -518,13 +522,13 @@ namespace Rstats {
         case Rstats::Type::COMPLEX : \
           v3 = Rstats::VectorFunc::new_complex(length); \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_complex_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_complex_value(v1, i), Rstats::VectorFunc::get_complex_value(v2, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Complex>(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i), Rstats::VectorFunc::get_value<Rstats::Complex>(v2, i))); \
           } \
           break; \
         case Rstats::Type::DOUBLE : \
           v3 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_double_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_double_value(v1, i), Rstats::VectorFunc::get_double_value(v2, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Double>(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Double>(v1, i), Rstats::VectorFunc::get_value<Rstats::Double>(v2, i))); \
           } \
           break; \
         case Rstats::Type::INTEGER : \
@@ -532,7 +536,7 @@ namespace Rstats {
           v3 = Rstats::VectorFunc::new_integer(length); \
           for (IV i = 0; i < length; i++) { \
             try {\
-              Rstats::VectorFunc::set_integer_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_integer_value(v1, i), Rstats::VectorFunc::get_integer_value(v2, i))); \
+              Rstats::VectorFunc::set_value<Rstats::Integer>(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i), Rstats::VectorFunc::get_value<Rstats::Integer>(v2, i))); \
             }\
             catch (const char* e) {\
               Rstats::VectorFunc::add_na_position(v3, i);\
@@ -562,20 +566,20 @@ namespace Rstats {
         case Rstats::Type::COMPLEX : \
           v3 = Rstats::VectorFunc::new_complex(length); \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_complex_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_complex_value(v1, i), Rstats::VectorFunc::get_complex_value(v2, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Complex>(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i), Rstats::VectorFunc::get_value<Rstats::Complex>(v2, i))); \
           } \
           break; \
         case Rstats::Type::DOUBLE : \
           v3 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_double_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_double_value(v1, i), Rstats::VectorFunc::get_double_value(v2, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Double>(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Double>(v1, i), Rstats::VectorFunc::get_value<Rstats::Double>(v2, i))); \
           } \
           break; \
         case Rstats::Type::INTEGER : \
         case Rstats::Type::LOGICAL : \
           v3 = Rstats::VectorFunc::new_double(length); \
           for (IV i = 0; i < length; i++) { \
-            Rstats::VectorFunc::set_double_value(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_integer_value(v1, i), Rstats::VectorFunc::get_integer_value(v2, i))); \
+            Rstats::VectorFunc::set_value<Rstats::Double>(v3, i, ELEMENT_FUNC_NAME(Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i), Rstats::VectorFunc::get_value<Rstats::Integer>(v2, i))); \
           } \
           break; \
         default: \
@@ -590,8 +594,8 @@ namespace Rstats {
   namespace VectorFunc {
     Rstats::Vector* new_vector();
     void delete_vector(Rstats::Vector*);
-    SV* get_value(Rstats::Vector*, IV);
-    SV* get_values(Rstats::Vector*);
+    SV* create_sv_value(Rstats::Vector*, IV);
+    SV* create_sv_values(Rstats::Vector*);
     
     bool is_character (Rstats::Vector*);
     bool is_complex (Rstats::Vector*);
@@ -600,72 +604,6 @@ namespace Rstats {
     bool is_numeric (Rstats::Vector*);
     bool is_logical (Rstats::Vector*);
     
-    template <typename T>
-    Rstats::Vector2<T>* new_vector2 (IV length) {
-      Rstats::Vector2<T>* v1 = new Rstats::Vector2<T>;
-      v1->values = new std::vector<T>(length);
-      v1->na_positions = new std::map<IV, IV>;
-      
-      return v1;
-    }
-    
-    template <typename T>
-    std::vector<T>* get_values2(Rstats::Vector2<T>* v1) {
-      return (std::vector<T>*)v1->values;
-    }
-    
-    template <typename T>
-    IV get_length2 (Rstats::Vector2<T>* v1) {
-      if (v1->values == NULL) {
-        return 0;
-      }
-      
-      return Rstats::VectorFunc::get_values2(v1)->size();
-    }
-    
-    template <typename T>
-    void set_value2(Rstats::Vector2<T>* v1, IV pos, T value) {
-      (*Rstats::VectorFunc::get_values2(v1))[pos] = value;
-    }
-    
-    template <typename T>
-    void add_na_position2(Rstats::Vector2<T>* v1, IV position) {
-      (*v1->na_positions)[position] = 1;
-    }
-    
-    template <typename T2, typename T1>
-    void merge_na_positions2(Rstats::Vector2<T2>* v2, Rstats::Vector2<T1>* v1) {
-      for(std::map<IV, IV>::iterator it = v1->na_positions->begin(); it != v1->na_positions->end(); ++it) {
-        Rstats::VectorFunc::add_na_position2(v2, it->first);
-      }
-    }
-    
-    template <typename T>
-    T get_value2(Rstats::Vector2<T>* v1, IV pos) {
-      return (*Rstats::VectorFunc::get_values2(v1))[pos];
-    }
-    
-    template <typename T_func, typename T_out, typename T_in>
-    Rstats::Vector2<T_out>* operate_un_math(T_func func, Rstats::Vector2<T_in>* v1) {
-      IV length = Rstats::VectorFunc::get_length2(v1);
-      Rstats::Vector2<T_out>* v2 = Rstats::VectorFunc::new_vector2<T_out>(length);
-      for (IV i = 0; i < length; i++) {
-        Rstats::VectorFunc::set_value2(v2, i, func(Rstats::VectorFunc::get_value2(v1, i)));
-      }
-      Rstats::VectorFunc::merge_na_positions2(v2, v1);
-      return v2;
-    }
-    
-    void sin2(Rstats::Vector2<SV*>* v1);
-    Rstats::Vector2<std::complex<NV> >* sin2(Rstats::Vector2<std::complex<NV> >* v1);
-    Rstats::Vector2<NV>* sin2(Rstats::Vector2<NV>* v1);
-    Rstats::Vector2<NV>* sin2(Rstats::Vector2<IV>* v1);
-        
-    std::vector<SV*>* get_character_values(Rstats::Vector*);
-    std::vector<std::complex<NV> >* get_complex_values(Rstats::Vector*);
-    std::vector<NV>* get_double_values(Rstats::Vector*);
-    std::vector<IV>* get_integer_values(Rstats::Vector*);
-    
     Rstats::Type::Enum get_type(Rstats::Vector*);
     void add_na_position(Rstats::Vector*, IV);
     bool exists_na_position(Rstats::Vector*, IV position);
@@ -675,23 +613,14 @@ namespace Rstats {
     
     Rstats::Vector* new_character(IV, SV*);
     Rstats::Vector* new_character(IV);
-    SV* get_character_value(Rstats::Vector*, IV);
-    void set_character_value(Rstats::Vector*, IV, SV*);
     Rstats::Vector* new_complex(IV);
     Rstats::Vector* new_complex(IV, std::complex<NV>);
-    std::complex<NV> get_complex_value(Rstats::Vector*, IV);
-    void set_complex_value(Rstats::Vector*, IV, std::complex<NV>);
     Rstats::Vector* new_double(IV);
     Rstats::Vector* new_double(IV, NV);
-    NV get_double_value(Rstats::Vector*, IV);
-    void set_double_value(Rstats::Vector*, IV, NV);
 
     Rstats::Vector* new_integer(IV);
     Rstats::Vector* new_integer(IV, IV);
     
-    IV get_integer_value(Rstats::Vector*, IV);
-    void set_integer_value(Rstats::Vector*, IV, IV);
-    
     Rstats::Vector* new_logical(IV);
     Rstats::Vector* new_logical(IV, IV);
     Rstats::Vector* new_true();
@@ -768,6 +697,25 @@ namespace Rstats {
     Rstats::Vector* prod(Rstats::Vector*);
     Rstats::Vector* sum(Rstats::Vector*);
     Rstats::Vector* clone(Rstats::Vector*);
+    
+    // T is std::vector<xxx>*
+    template<class T>
+    std::vector<T>* get_values(Rstats::Vector* v1) {
+      return (std::vector<T>*)v1->values;
+    }
+    template <class T>
+    T get_value(Rstats::Vector* v1, IV pos) {
+      return (*Rstats::VectorFunc::get_values<T>(v1))[pos];
+    }
+    template <>
+    Rstats::Character get_value<Rstats::Character>(Rstats::Vector* v1, IV pos);
+    
+    template<class T>
+    void set_value(Rstats::Vector* v1, IV pos, T value) {
+      (*Rstats::VectorFunc::get_values<T>(v1))[pos] = value;
+    }
+    template <>
+    void set_value<Rstats::Character>(Rstats::Vector* v1, IV pos, Rstats::Character value);
   }
   
   // Rstats::Func
@@ -809,7 +757,6 @@ namespace Rstats {
     SV* as_vector(SV*, SV*);
     SV* new_data_frame(SV*);
     SV* new_list(SV*);
-    SV* new_vector(SV*, SV*, SV*);
     SV* copy_attrs_to(SV*, SV*, SV*);
     SV* copy_attrs_to(SV*, SV*, SV*, SV*);
 
@@ -901,6 +848,8 @@ namespace Rstats {
     SV* not_equal(SV*, SV*, SV*);
     SV* And(SV*, SV*, SV*);
     SV* Or(SV*, SV*, SV*);
+
+    SV* sin(SV*, SV*);
   }
 }
 
