@@ -4,6 +4,33 @@
 namespace Rstats {
   namespace Func {
 
+    SV* sin(SV* sv_r, SV* sv_x1) {
+
+      sv_x1 = Rstats::Func::to_c(sv_r, sv_x1);
+      SV* sv_x2 = Rstats::Func::operate_unary(sv_r, &Rstats::VectorFunc::sin, sv_x1);
+      
+      return sv_x2;
+    }
+
+    SV* operate_unary(SV* sv_r, Rstats::Vector* (*func)(Rstats::Vector*), SV* sv_x1) {
+      
+      Rstats::Vector* x2_elements = (*func)(get_vector(sv_r, sv_x1));
+      SV* sv_x2 = Rstats::Func::new_null(sv_r);
+      set_vector(sv_r, sv_x2, x2_elements);
+      Rstats::Func::copy_attrs_to(sv_r, sv_x1, sv_x2);
+      
+      return sv_x2;
+    }
+    
+    SV* atan2(SV* sv_r, SV* sv_x1, SV* sv_x2) {
+      sv_x1 = Rstats::Func::to_c(sv_r, sv_x1);
+      sv_x2 = Rstats::Func::to_c(sv_r, sv_x2);
+      
+      SV* sv_x3 = Rstats::Func::operate_binary(sv_r, &Rstats::VectorFunc::atan2, sv_x1, sv_x2);
+      
+      return sv_x3;
+    }
+
     SV* operate_binary(SV* sv_r, Rstats::Vector* (*func)(Rstats::Vector*, Rstats::Vector*), SV* sv_x1, SV* sv_x2) {
       
       sv_x1 = Rstats::Func::to_c(sv_r, sv_x1);
@@ -61,7 +88,8 @@ namespace Rstats {
       for (IV i = 0; i < element_length; i++) {
         Rstats::Type::Enum type;
         SV* sv_element = Rstats::pl_av_fetch(sv_elements, i);
-        if (sv_isobject(sv_element) && sv_derived_from(sv_element, "Rstats::Array")) {
+        
+        if (to_bool(sv_r, Rstats::Func::is_vector(sv_r, sv_element)) || to_bool(sv_r, Rstats::Func::is_array(sv_r, sv_element))) {
           length += Rstats::VectorFunc::get_length(Rstats::Func::get_vector(sv_r, sv_element));
           type = Rstats::VectorFunc::get_type(Rstats::Func::get_vector(sv_r, sv_element));
           type_h[type] = 1;
@@ -105,10 +133,10 @@ namespace Rstats {
       IV pos = 0;
       for (IV i = 0; i < element_length; i++) {
         SV* sv_element = Rstats::pl_av_fetch(sv_elements, i);
-        if (sv_derived_from(sv_element, "Rstats::Array")) {
+        if (to_bool(sv_r, Rstats::Func::is_vector(sv_r, sv_element)) || to_bool(sv_r, Rstats::Func::is_array(sv_r, sv_element))) {
           
           Rstats::Vector* v1;
-          if (sv_derived_from(sv_element, "Rstats::Array")) {
+          if (to_bool(sv_r, Rstats::Func::is_vector(sv_r, sv_element)) || to_bool(sv_r, Rstats::Func::is_array(sv_r, sv_element))) {
             v1 = Rstats::Func::get_vector(sv_r, sv_element);
           }
           else {
@@ -143,19 +171,19 @@ namespace Rstats {
             }
             else {
               if (type == Rstats::Type::CHARACTER) {
-                Rstats::VectorFunc::set_character_value(v2, pos, Rstats::VectorFunc::get_character_value(v_tmp, k));
+                Rstats::VectorFunc::set_value<Rstats::Character>(v2, pos, Rstats::VectorFunc::get_value<Rstats::Character>(v_tmp, k));
               }
               else if (type == Rstats::Type::COMPLEX) {
-                Rstats::VectorFunc::set_complex_value(v2, pos, Rstats::VectorFunc::get_complex_value(v_tmp, k));
+                Rstats::VectorFunc::set_value<Rstats::Complex>(v2, pos, Rstats::VectorFunc::get_value<Rstats::Complex>(v_tmp, k));
               }
               else if (type == Rstats::Type::DOUBLE) {
-                Rstats::VectorFunc::set_double_value(v2, pos, Rstats::VectorFunc::get_double_value(v_tmp, k));
+                Rstats::VectorFunc::set_value<Rstats::Double>(v2, pos, Rstats::VectorFunc::get_value<Rstats::Double>(v_tmp, k));
               }
               else if (type == Rstats::Type::INTEGER) {
-                Rstats::VectorFunc::set_integer_value(v2, pos, Rstats::VectorFunc::get_integer_value(v_tmp, k));
+                Rstats::VectorFunc::set_value<Rstats::Integer>(v2, pos, Rstats::VectorFunc::get_value<Rstats::Integer>(v_tmp, k));
               }
               else {
-                Rstats::VectorFunc::set_integer_value(v2, pos, Rstats::VectorFunc::get_integer_value(v_tmp, k));
+                Rstats::VectorFunc::set_value<Rstats::Integer>(v2, pos, Rstats::VectorFunc::get_value<Rstats::Integer>(v_tmp, k));
               }
             }
             
@@ -169,19 +197,19 @@ namespace Rstats {
         else {
           if (SvOK(sv_element)) {
             if (type == Rstats::Type::CHARACTER) {
-              Rstats::VectorFunc::set_character_value(v2, pos, sv_element);
+              Rstats::VectorFunc::set_value<Rstats::Character>(v2, pos, sv_element);
             }
             else if (type == Rstats::Type::COMPLEX) {
-              Rstats::VectorFunc::set_complex_value(v2, pos, std::complex<NV>(SvNV(sv_element), 0));
+              Rstats::VectorFunc::set_value<Rstats::Complex>(v2, pos, std::complex<NV>(SvNV(sv_element), 0));
             }
             else if (type == Rstats::Type::DOUBLE) {
-              Rstats::VectorFunc::set_double_value(v2, pos, SvNV(sv_element));
+              Rstats::VectorFunc::set_value<Rstats::Double>(v2, pos, SvNV(sv_element));
             }
             else if (type == Rstats::Type::INTEGER) {
-              Rstats::VectorFunc::set_integer_value(v2, pos, SvIV(sv_element));
+              Rstats::VectorFunc::set_value<Rstats::Integer>(v2, pos, SvIV(sv_element));
             }
             else {
-              Rstats::VectorFunc::set_integer_value(v2, pos, SvIV(sv_element));
+              Rstats::VectorFunc::set_value<Rstats::Integer>(v2, pos, SvIV(sv_element));
             }
           }
           else {
@@ -334,20 +362,10 @@ namespace Rstats {
       return sv_new_xs;
     }
     
-    SV* operate_unary(SV* sv_r, Rstats::Vector* (*func)(Rstats::Vector*), SV* sv_x1) {
-      
-      Rstats::Vector* x2_elements = (*func)(get_vector(sv_r, sv_x1));
-      SV* sv_x2 = Rstats::Func::new_null(sv_r);
-      set_vector(sv_r, sv_x2, x2_elements);
-      Rstats::Func::copy_attrs_to(sv_r, sv_x1, sv_x2);
-      
-      return sv_x2;
-    }
-    
     SV* is_matrix(SV* sv_r, SV* sv_x1) {
 
       bool is = sv_isobject(sv_x1)
-        && sv_derived_from(sv_x1, "Rstats::Array")
+        && sv_derived_from(sv_x1, "Rstats::Object")
         && SvIV(length_value(sv_r, dim(sv_r, sv_x1))) == 2;
       
       SV* sv_is = is ? Rstats::Func::new_true(sv_r) : Rstats::Func::new_false(sv_r);
@@ -394,9 +412,9 @@ namespace Rstats {
     }
 
     SV* values(SV* sv_r, SV* sv_x1) {
-      Rstats::Vector* x1 = get_vector(sv_r, sv_x1);
+      Rstats::Vector* v1 = get_vector(sv_r, sv_x1);
       
-      SV* sv_values = Rstats::VectorFunc::get_values(x1);
+      SV* sv_values = Rstats::VectorFunc::create_sv_values(v1);
       
       return sv_values;
     }
@@ -450,7 +468,8 @@ namespace Rstats {
       
       bool is =
         sv_isobject(sv_x1)
-        && sv_derived_from(sv_x1, "Rstats::Array")
+        && sv_derived_from(sv_x1, "Rstats::Object")
+        && strEQ(SvPV_nolen(Rstats::pl_hv_fetch(sv_x1, "object_type")), "array")
         && !Rstats::pl_hv_exists(sv_x1, "dim");
       
       SV* sv_is = is ? Rstats::pl_new_sv_iv(1) : Rstats::pl_new_sv_iv(0);
@@ -465,7 +484,7 @@ namespace Rstats {
       
       Rstats::Vector* v1 = get_vector(sv_r, sv_x1);
       if (v1->type == Rstats::Type::LOGICAL) {
-        IV is = Rstats::VectorFunc::get_integer_value(v1, 0);
+        IV is = Rstats::VectorFunc::get_value<Rstats::Integer>(v1, 0);
         return is;
       }
       else {
@@ -476,7 +495,8 @@ namespace Rstats {
     SV* is_array(SV* sv_r, SV* sv_x1) {
 
       bool is = sv_isobject(sv_x1)
-        && sv_derived_from(sv_x1, "Rstats::Array")
+        && sv_derived_from(sv_x1, "Rstats::Object")
+        && strEQ(SvPV_nolen(Rstats::pl_hv_fetch(sv_x1, "object_type")), "array")
         && Rstats::pl_hv_exists(sv_x1, "dim");
       
       SV* sv_is = is ? Rstats::Func::new_true(sv_r) : Rstats::Func::new_false(sv_r);
@@ -496,8 +516,9 @@ namespace Rstats {
     SV* new_array(SV* sv_r) {
       
       SV* sv_array = Rstats::pl_new_hv_ref();
-      sv_bless(sv_array, gv_stashpv("Rstats::Array", 1));
+      sv_bless(sv_array, gv_stashpv("Rstats::Object", 1));
       Rstats::pl_hv_store(sv_array, "r", sv_r);
+      Rstats::pl_hv_store(sv_array, "object_type", Rstats::pl_new_sv_pv("array"));
       
       return sv_array;
     }
@@ -518,7 +539,7 @@ namespace Rstats {
         SV* sv_value = Rstats::pl_av_fetch(sv_values, i);
 
         if (SvOK(sv_value)) {
-          Rstats::VectorFunc::set_character_value(
+          Rstats::VectorFunc::set_value<Rstats::Character>(
             v1,
             i,
             sv_value
@@ -552,17 +573,17 @@ namespace Rstats {
         if (SvOK(sv_value)) {
           char* sv_value_str = SvPV_nolen(sv_value);
           if (strEQ(sv_value_str, "NaN")) {
-            Rstats::VectorFunc::set_double_value(v1, i, NAN);
+            Rstats::VectorFunc::set_value<Rstats::Double>(v1, i, NAN);
           }
           else if (strEQ(sv_value_str, "Inf")) {
-            Rstats::VectorFunc::set_double_value(v1, i, INFINITY);
+            Rstats::VectorFunc::set_value<Rstats::Double>(v1, i, INFINITY);
           }
           else if (strEQ(sv_value_str, "-Inf")) {
-            Rstats::VectorFunc::set_double_value(v1, i, -(INFINITY));
+            Rstats::VectorFunc::set_value<Rstats::Double>(v1, i, -(INFINITY));
           }
           else {
             NV value = SvNV(sv_value);
-            Rstats::VectorFunc::set_double_value(v1, i, value);
+            Rstats::VectorFunc::set_value<Rstats::Double>(v1, i, value);
           }
         }
         else {
@@ -635,7 +656,7 @@ namespace Rstats {
             im = 0;
           }
           
-          Rstats::VectorFunc::set_complex_value(
+          Rstats::VectorFunc::set_value<Rstats::Complex>(
             v1,
             i,
             std::complex<NV>(re, im)
@@ -667,7 +688,7 @@ namespace Rstats {
         SV* sv_value = Rstats::pl_av_fetch(sv_values, i);
         
         if (SvOK(sv_value)) {
-          Rstats::VectorFunc::set_integer_value(
+          Rstats::VectorFunc::set_value<Rstats::Integer>(
             v1,
             i,
             SvIV(sv_value)
@@ -703,7 +724,7 @@ namespace Rstats {
         SV* sv_value = Rstats::pl_av_fetch(sv_values, i);
         
         if (SvOK(sv_value)) {
-          Rstats::VectorFunc::set_integer_value(
+          Rstats::VectorFunc::set_value<Rstats::Integer>(
             v1,
             i,
             SvIV(sv_value)
@@ -853,7 +874,8 @@ namespace Rstats {
     SV* is_data_frame(SV* sv_r, SV* sv_x1) {
       
       bool is = sv_isobject(sv_x1)
-        && sv_derived_from(sv_x1, "Rstats::DataFrame");
+        && sv_derived_from(sv_x1, "Rstats::Object")
+        && strEQ(SvPV_nolen(Rstats::pl_hv_fetch(sv_x1, "object_type")), "data.frame");
         
       SV* sv_x_is = is ? new_true(sv_r) : new_false(sv_r);
       
@@ -863,7 +885,8 @@ namespace Rstats {
     SV* is_list(SV* sv_r, SV* sv_x1) {
       
       bool is = sv_isobject(sv_x1)
-        && sv_derived_from(sv_x1, "Rstats::List");
+        && sv_derived_from(sv_x1, "Rstats::Object")
+        && strEQ(SvPV_nolen(Rstats::pl_hv_fetch(sv_x1, "object_type")), "list");
         
       SV* sv_x_is = is ? new_true(sv_r) : new_false(sv_r);
       
@@ -888,42 +911,20 @@ namespace Rstats {
 
     SV* new_data_frame(SV* sv_r) {
       SV* sv_data_frame = Rstats::pl_new_hv_ref();
-      Rstats::pl_sv_bless(sv_data_frame, "Rstats::DataFrame");
+      Rstats::pl_sv_bless(sv_data_frame, "Rstats::Object");
       Rstats::pl_hv_store(sv_data_frame, "r", sv_r);
+      Rstats::pl_hv_store(sv_data_frame, "object_type", Rstats::pl_new_sv_pv("data.frame"));
       
       return sv_data_frame;
     }
 
     SV* new_list(SV* sv_r) {
-      SV* sv_data_frame = Rstats::pl_new_hv_ref();
-      Rstats::pl_sv_bless(sv_data_frame, "Rstats::List");
-      Rstats::pl_hv_store(sv_data_frame, "r", sv_r);
+      SV* sv_list = Rstats::pl_new_hv_ref();
+      Rstats::pl_sv_bless(sv_list, "Rstats::Object");
+      Rstats::pl_hv_store(sv_list, "r", sv_r);
+      Rstats::pl_hv_store(sv_list, "object_type", Rstats::pl_new_sv_pv("list"));
       
-      return sv_data_frame;
-    }
-
-    SV* new_vector(SV* sv_r, SV* sv_type, SV* sv_values) {
-      
-      char* type = SvPV_nolen(sv_type);
-      
-      if (strEQ(type, "character")) {
-        return new_character(sv_r, sv_values);
-      }
-      else if (strEQ(type, "complex")) {
-        return new_complex(sv_r, sv_values);
-      }
-      else if (strEQ(type, "double")) {
-        return new_double(sv_r, sv_values);
-      }
-      else if (strEQ(type, "integer")) {
-        return new_integer(sv_r, sv_values);
-      }
-      else if (strEQ(type, "logical")) {
-        return new_logical(sv_r, sv_values);
-      }
-      else {
-        croak("Invalid type %s is passed(new_vector)", type);
-      }
+      return sv_list;
     }
 
     SV* copy_attrs_to(SV* sv_r, SV* sv_x1, SV* sv_x2) {
@@ -1121,10 +1122,10 @@ namespace Rstats {
       
       for (IV i = 0; i < Rstats::VectorFunc::get_length(x1); i++) {
         if (Rstats::VectorFunc::exists_na_position(x1, i)) {
-          Rstats::VectorFunc::set_integer_value(x2, i, 1);
+          Rstats::VectorFunc::set_value<Rstats::Integer>(x2, i, 1);
         }
         else {
-          Rstats::VectorFunc::set_integer_value(x2, i, 0);
+          Rstats::VectorFunc::set_value<Rstats::Integer>(x2, i, 0);
         }
       }
       
@@ -1193,7 +1194,7 @@ namespace Rstats {
       
       IV match = 0;
       for (IV i = 0; i < v_classes_length; i++) {
-        SV* sv_class = Rstats::VectorFunc::get_character_value(v_classes, i);
+        SV* sv_class = Rstats::VectorFunc::get_value<Rstats::Character>(v_classes, i);
         if (strEQ(SvPV_nolen(sv_class), "factor")) {
           match = 1;
           break;
@@ -1211,7 +1212,7 @@ namespace Rstats {
       
       IV match = 0;
       for (IV i = 0; i < v_classes_length; i++) {
-        SV* sv_class = Rstats::VectorFunc::get_character_value(v_classes, i);
+        SV* sv_class = Rstats::VectorFunc::get_value<Rstats::Character>(v_classes, i);
         if (strEQ(SvPV_nolen(sv_class), "ordered")) {
           match = 1;
           break;
@@ -1254,7 +1255,7 @@ namespace Rstats {
         if (Rstats::VectorFunc::is_character(v1)) {
           for (IV i = 0; i < length; i++) {
             Rstats::Vector* v2
-              = Rstats::VectorFunc::new_character(1, Rstats::VectorFunc::get_character_value(v1, i));
+              = Rstats::VectorFunc::new_character(1, Rstats::VectorFunc::get_value<Rstats::Character>(v1, i));
             if (Rstats::VectorFunc::exists_na_position(v1, i)) {
               Rstats::VectorFunc::add_na_position(v2, 0);
             }
@@ -1266,7 +1267,7 @@ namespace Rstats {
         else if (Rstats::VectorFunc::is_complex(v1)) {
           for (IV i = 0; i < length; i++) {
             Rstats::Vector* v2
-              = Rstats::VectorFunc::new_complex(1, Rstats::VectorFunc::get_complex_value(v1, i));
+              = Rstats::VectorFunc::new_complex(1, Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i));
             if (Rstats::VectorFunc::exists_na_position(v1, i)) {
               Rstats::VectorFunc::add_na_position(v2, 0);
             }
@@ -1278,7 +1279,7 @@ namespace Rstats {
         else if (Rstats::VectorFunc::is_double(v1)) {
           for (IV i = 0; i < length; i++) {
             Rstats::Vector* v2
-              = Rstats::VectorFunc::new_double(1, Rstats::VectorFunc::get_double_value(v1, i));
+              = Rstats::VectorFunc::new_double(1, Rstats::VectorFunc::get_value<Rstats::Double>(v1, i));
             if (Rstats::VectorFunc::exists_na_position(v1, i)) {
               Rstats::VectorFunc::add_na_position(v2, 0);
             }
@@ -1290,7 +1291,7 @@ namespace Rstats {
         else if (Rstats::VectorFunc::is_integer(v1)) {
           for (IV i = 0; i < length; i++) {
             Rstats::Vector* v2
-              = Rstats::VectorFunc::new_integer(1, Rstats::VectorFunc::get_integer_value(v1, i));
+              = Rstats::VectorFunc::new_integer(1, Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i));
             if (Rstats::VectorFunc::exists_na_position(v1, i)) {
               Rstats::VectorFunc::add_na_position(v2, 0);
             }
@@ -1302,7 +1303,7 @@ namespace Rstats {
         else if (Rstats::VectorFunc::is_logical(v1)) {
           for (IV i = 0; i < length; i++) {
             Rstats::Vector* v2
-              = Rstats::VectorFunc::new_logical(1, Rstats::VectorFunc::get_integer_value(v1, i));
+              = Rstats::VectorFunc::new_logical(1, Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i));
             if (Rstats::VectorFunc::exists_na_position(v1, i)) {
               Rstats::VectorFunc::add_na_position(v2, 0);
             }
@@ -1336,7 +1337,7 @@ namespace Rstats {
             na_positions.push_back(i);
           }
           else {
-            Rstats::VectorFunc::set_character_value(compose_elements, i, Rstats::VectorFunc::get_character_value(element, 0));
+            Rstats::VectorFunc::set_value<Rstats::Character>(compose_elements, i, Rstats::VectorFunc::get_value<Rstats::Character>(element, 0));
           }
         }
       }
@@ -1353,7 +1354,7 @@ namespace Rstats {
             na_positions.push_back(i);
           }
           else {
-           Rstats::VectorFunc::set_complex_value(compose_elements, i, Rstats::VectorFunc::get_complex_value(element, 0));
+           Rstats::VectorFunc::set_value<Rstats::Complex>(compose_elements, i, Rstats::VectorFunc::get_value<Rstats::Complex>(element, 0));
           }
         }
       }
@@ -1370,13 +1371,13 @@ namespace Rstats {
             na_positions.push_back(i);
           }
           else {
-            Rstats::VectorFunc::set_double_value(compose_elements, i, Rstats::VectorFunc::get_double_value(element, 0));
+            Rstats::VectorFunc::set_value<Rstats::Double>(compose_elements, i, Rstats::VectorFunc::get_value<Rstats::Double>(element, 0));
           }
         }
       }
       else if (strEQ(mode, "integer")) {
         compose_elements = Rstats::VectorFunc::new_integer(len);
-        std::vector<IV>* values = Rstats::VectorFunc::get_integer_values(compose_elements);
+        std::vector<IV>* values = Rstats::VectorFunc::get_values<Rstats::Integer>(compose_elements);
         for (IV i = 0; i < len; i++) {
           SV* sv_x1 = Rstats::pl_av_fetch(sv_elements, i);
           if (!SvOK(sv_x1)) {
@@ -1388,13 +1389,13 @@ namespace Rstats {
             na_positions.push_back(i);
           }
           else {
-            Rstats::VectorFunc::set_integer_value(compose_elements, i, Rstats::VectorFunc::get_integer_value(element, 0));
+            Rstats::VectorFunc::set_value<Rstats::Integer>(compose_elements, i, Rstats::VectorFunc::get_value<Rstats::Integer>(element, 0));
           }
         }
       }
       else if (strEQ(mode, "logical")) {
         compose_elements = Rstats::VectorFunc::new_logical(len);
-        std::vector<IV>* values = Rstats::VectorFunc::get_integer_values(compose_elements);
+        std::vector<IV>* values = Rstats::VectorFunc::get_values<Rstats::Integer>(compose_elements);
         for (IV i = 0; i < len; i++) {
           SV* sv_x1 = Rstats::pl_av_fetch(sv_elements, i);
           if (!SvOK(sv_x1)) {
@@ -1406,7 +1407,7 @@ namespace Rstats {
             na_positions.push_back(i);
           }
           else {
-            Rstats::VectorFunc::set_integer_value(compose_elements, i, Rstats::VectorFunc::get_integer_value(element, 0));
+            Rstats::VectorFunc::set_value<Rstats::Integer>(compose_elements, i, Rstats::VectorFunc::get_value<Rstats::Integer>(element, 0));
           }
         }
       }
@@ -1614,7 +1615,7 @@ namespace Rstats {
     }
 
     SV* length(SV* sv_r, SV* sv_container) {
-      if (sv_isobject(sv_container) && sv_derived_from(sv_container, "Rstats::Array")) {
+      if (to_bool(sv_r, Rstats::Func::is_vector(sv_r, sv_container)) || to_bool(sv_r, Rstats::Func::is_array(sv_r, sv_container))) {
         return Rstats::Func::new_integer(
           sv_r,
           Rstats::VectorFunc::get_length(Rstats::Func::get_vector(sv_r, sv_container))
