@@ -10,22 +10,6 @@ namespace Rstats {
 
     RSTATS_DEF_VECTOR_FUNC_UN_MATH(negation, Rstats::ElementFunc::negation)
 
-    // sin2
-    void sin2(Rstats::Vector2<SV*>* v1) {
-      croak("Error in sin() : invalid argument to sin()");
-    }
-    Rstats::Vector2<std::complex<NV> >* sin2(Rstats::Vector2<std::complex<NV> >* v1) {
-      return Rstats::VectorFunc::operate_un_math<
-          std::complex<NV>  (*)(std::complex<NV> ), std::complex<NV> , std::complex<NV>
-        >(&Rstats::ElementFunc::sin, v1);
-    }
-    Rstats::Vector2<NV>* sin2(Rstats::Vector2<NV>* v1) {
-      return Rstats::VectorFunc::operate_un_math<NV (*)(NV), NV, NV>(&Rstats::ElementFunc::sin, v1);
-    }
-    Rstats::Vector2<NV>* sin2(Rstats::Vector2<IV>* v1) {
-      return Rstats::VectorFunc::operate_un_math<NV (*)(IV), NV, IV>(&Rstats::ElementFunc::sin, v1);
-    }
-
     RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(sin, Rstats::ElementFunc::sin)
     RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(cos, Rstats::ElementFunc::cos)
     RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(tan, Rstats::ElementFunc::tan)
@@ -70,14 +54,24 @@ namespace Rstats {
     RSTATS_DEF_VECTOR_FUNC_BIN_MATH_INTEGER_TO_DOUBLE(divide, Rstats::ElementFunc::divide)
     RSTATS_DEF_VECTOR_FUNC_BIN_MATH_INTEGER_TO_DOUBLE(atan2, Rstats::ElementFunc::atan2)
     RSTATS_DEF_VECTOR_FUNC_BIN_MATH_INTEGER_TO_DOUBLE(pow, Rstats::ElementFunc::pow)
-
-
-    SV* get_values(Rstats::Vector* v1) {
+    
+    template <>
+    Rstats::Character get_value<Rstats::Character>(Rstats::Vector* v1, IV pos) {
+      Rstats::Character value = (*Rstats::VectorFunc::get_values<Rstats::Character>(v1))[pos];
+      if (value == NULL) {
+        return NULL;
+      }
+      else {
+        return Rstats::pl_new_sv_sv(value);
+      }
+    }
+    
+    SV* create_sv_values(Rstats::Vector* v1) {
       
       IV length = Rstats::VectorFunc::get_length(v1);
       SV* sv_values = Rstats::pl_new_av_ref();
       for (IV i = 0; i < length; i++) {
-        Rstats::pl_av_push(sv_values, Rstats::VectorFunc::get_value(v1, i));
+        Rstats::pl_av_push(sv_values, Rstats::VectorFunc::create_sv_value(v1, i));
       }
       
       return sv_values;
@@ -107,22 +101,6 @@ namespace Rstats {
       return Rstats::VectorFunc::get_type(v1) == Rstats::Type::LOGICAL;
     }
 
-    std::vector<SV*>* get_character_values(Rstats::Vector* v1) {
-      return (std::vector<SV*>*)v1->values;
-    }
-
-    std::vector<std::complex<NV> >* get_complex_values(Rstats::Vector* v1) {
-      return (std::vector<std::complex<NV> >*)v1->values;
-    }
-
-    std::vector<NV>* get_double_values(Rstats::Vector* v1) {
-      return (std::vector<NV>*)v1->values;
-    }
-
-    std::vector<IV>* get_integer_values(Rstats::Vector* v1) {
-      return (std::vector<IV>*)v1->values;
-    }
-
     Rstats::Type::Enum get_type(Rstats::Vector* v1) {
       return v1->type;
     }
@@ -153,17 +131,17 @@ namespace Rstats {
       Rstats::Type::Enum type = Rstats::VectorFunc::get_type(v1);
       switch (type) {
         case Rstats::Type::CHARACTER :
-          return Rstats::VectorFunc::get_character_values(v1)->size();
+          return Rstats::VectorFunc::get_values<Rstats::Character>(v1)->size();
           break;
         case Rstats::Type::COMPLEX :
-          return Rstats::VectorFunc::get_complex_values(v1)->size();
+          return Rstats::VectorFunc::get_values<Rstats::Complex>(v1)->size();
           break;
         case Rstats::Type::DOUBLE :
-          return Rstats::VectorFunc::get_double_values(v1)->size();
+          return Rstats::VectorFunc::get_values<Rstats::Double>(v1)->size();
           break;
         case Rstats::Type::INTEGER :
         case Rstats::Type::LOGICAL :
-          return Rstats::VectorFunc::get_integer_values(v1)->size();
+          return Rstats::VectorFunc::get_values<Rstats::Integer>(v1)->size();
           break;
       }
     }
@@ -183,7 +161,7 @@ namespace Rstats {
       if (v1->values != NULL){ 
         switch (type) {
           case Rstats::Type::CHARACTER : {
-            std::vector<SV*>* values = Rstats::VectorFunc::get_character_values(v1);
+            std::vector<SV*>* values = Rstats::VectorFunc::get_values<Rstats::Character>(v1);
             for (IV i = 0; i < length; i++) {
               if ((*values)[i] != NULL) {
                 SvREFCNT_dec((*values)[i]);
@@ -193,18 +171,18 @@ namespace Rstats {
             break;
           }
           case Rstats::Type::COMPLEX : {
-            std::vector<std::complex<NV> >* values = Rstats::VectorFunc::get_complex_values(v1);
+            std::vector<std::complex<NV> >* values = Rstats::VectorFunc::get_values<Rstats::Complex>(v1);
             delete values;
             break;
           }
           case Rstats::Type::DOUBLE : {
-            std::vector<NV>* values = Rstats::VectorFunc::get_double_values(v1);
+            std::vector<NV>* values = Rstats::VectorFunc::get_values<Rstats::Double>(v1);
             delete values;
             break;
           }
           case Rstats::Type::INTEGER :
           case Rstats::Type::LOGICAL : {
-            std::vector<IV>* values = Rstats::VectorFunc::get_integer_values(v1);
+            std::vector<IV>* values = Rstats::VectorFunc::get_values<Rstats::Integer>(v1);
             delete values;
           }
         }
@@ -226,29 +204,20 @@ namespace Rstats {
 
       Rstats::Vector* v1 = Rstats::VectorFunc::new_character(length);
       for (IV i = 0; i < length; i++) {
-        Rstats::VectorFunc::set_character_value(v1, i, sv_str);
+        Rstats::VectorFunc::set_value<Rstats::Character>(v1, i, sv_str);
       }
       
       return v1;
     }
-
-    SV* get_character_value(Rstats::Vector* v1, IV pos) {
-      SV* value = (*Rstats::VectorFunc::get_character_values(v1))[pos];
-      if (value == NULL) {
-        return NULL;
-      }
-      else {
-        return Rstats::pl_new_sv_sv(value);
-      }
-    }
-
-    void set_character_value(Rstats::Vector* v1, IV pos, SV* value) {
+    
+    template <>
+    void set_value<Rstats::Character>(Rstats::Vector* v1, IV pos, Rstats::Character value) {
       if (value != NULL) {
-        SvREFCNT_dec((*Rstats::VectorFunc::get_character_values(v1))[pos]);
+        SvREFCNT_dec((*Rstats::VectorFunc::get_values<Rstats::Character>(v1))[pos]);
       }
       
       SV* new_value = Rstats::pl_new_sv_sv(value);
-      (*Rstats::VectorFunc::get_character_values(v1))[pos] = SvREFCNT_inc(new_value);
+      (*Rstats::VectorFunc::get_values<Rstats::Character>(v1))[pos] = SvREFCNT_inc(new_value);
     }
 
     Rstats::Vector* new_complex(IV length) {
@@ -264,19 +233,11 @@ namespace Rstats {
       
       Rstats::Vector* v1 = new_complex(length);
       for (IV i = 0; i < length; i++) {
-        Rstats::VectorFunc::set_complex_value(v1, i, z);
+        Rstats::VectorFunc::set_value<Rstats::Complex>(v1, i, z);
       }
       return v1;
     }
 
-    std::complex<NV> get_complex_value(Rstats::Vector* v1, IV pos) {
-      return (*Rstats::VectorFunc::get_complex_values(v1))[pos];
-    }
-
-    void set_complex_value(Rstats::Vector* v1, IV pos, std::complex<NV> value) {
-      (*Rstats::VectorFunc::get_complex_values(v1))[pos] = value;
-    }
-
     Rstats::Vector* new_double(IV length) {
       Rstats::Vector* v1 = new_vector();
       v1->values = new std::vector<NV>(length);
@@ -288,19 +249,11 @@ namespace Rstats {
     Rstats::Vector* new_double(IV length, NV value) {
       Rstats::Vector* v1 = new_double(length);
       for (IV i = 0; i < length; i++) {
-        Rstats::VectorFunc::set_double_value(v1, i, value);
+        Rstats::VectorFunc::set_value<Rstats::Double>(v1, i, value);
       }
       return v1;
     }
 
-    NV get_double_value(Rstats::Vector* v1, IV pos) {
-      return (*Rstats::VectorFunc::get_double_values(v1))[pos];
-    }
-
-    void set_double_value(Rstats::Vector* v1, IV pos, NV value) {
-      (*Rstats::VectorFunc::get_double_values(v1))[pos] = value;
-    }
-
     Rstats::Vector* new_integer(IV length) {
       
       Rstats::Vector* v1 = new_vector();
@@ -314,19 +267,11 @@ namespace Rstats {
       
       Rstats::Vector* v1 = new_integer(length);;
       for (IV i = 0; i < length; i++) {
-        Rstats::VectorFunc::set_integer_value(v1, i, value);
+        Rstats::VectorFunc::set_value<Rstats::Integer>(v1, i, value);
       }
       return v1;
     }
 
-    IV get_integer_value(Rstats::Vector* v1, IV pos) {
-      return (*Rstats::VectorFunc::get_integer_values(v1))[pos];
-    }
-
-    void set_integer_value(Rstats::Vector* v1, IV pos, IV value) {
-      (*Rstats::VectorFunc::get_integer_values(v1))[pos] = value;
-    }
-
     Rstats::Vector* new_logical(IV length) {
       Rstats::Vector* v1 = new_vector();
 
@@ -339,7 +284,7 @@ namespace Rstats {
     Rstats::Vector* new_logical(IV length, IV value) {
       Rstats::Vector* v1 = new_logical(length);
       for (IV i = 0; i < length; i++) {
-        Rstats::VectorFunc::set_integer_value(v1, i, value);
+        Rstats::VectorFunc::set_value<Rstats::Integer>(v1, i, value);
       }
       return v1;
     }
@@ -419,10 +364,10 @@ namespace Rstats {
       else {
         switch (type) {
           case Rstats::Type::CHARACTER :
-            sv_str = Rstats::VectorFunc::get_character_value(v1, pos);
+            sv_str = Rstats::VectorFunc::get_value<Rstats::Character>(v1, pos);
             break;
           case Rstats::Type::COMPLEX : {
-            std::complex<NV> z = Rstats::VectorFunc::get_complex_value(v1, pos);
+            std::complex<NV> z = Rstats::VectorFunc::get_value<Rstats::Complex>(v1, pos);
             NV re = z.real();
             NV im = z.imag();
             
@@ -460,7 +405,7 @@ namespace Rstats {
             break;
           }
           case Rstats::Type::DOUBLE : {
-            NV value = Rstats::VectorFunc::get_double_value(v1, pos);
+            NV value = Rstats::VectorFunc::get_value<Rstats::Double>(v1, pos);
             if (std::isinf(value) && value > 0) {
               sv_str = Rstats::pl_new_sv_pv("Inf");
             }
@@ -477,11 +422,11 @@ namespace Rstats {
             break;
           }
           case Rstats::Type::INTEGER :
-            sv_str = Rstats::pl_new_sv_iv(Rstats::VectorFunc::get_integer_value(v1, pos));
+            sv_str = Rstats::pl_new_sv_iv(Rstats::VectorFunc::get_value<Rstats::Integer>(v1, pos));
             sv_catpv(sv_str, "");
             break;
           case Rstats::Type::LOGICAL :
-            sv_str = Rstats::VectorFunc::get_integer_value(v1, pos)
+            sv_str = Rstats::VectorFunc::get_value<Rstats::Integer>(v1, pos)
               ? Rstats::pl_new_sv_pv("TRUE") : Rstats::pl_new_sv_pv("FALSE");
             break;
           default:
@@ -520,13 +465,13 @@ namespace Rstats {
       switch (type) {
         case Rstats::Type::CHARACTER :
           for (IV i = 0; i < length; i++) {
-            SV* sv_value = Rstats::VectorFunc::get_character_value(v1, i);
-            Rstats::VectorFunc::set_character_value(v2, i, sv_value);
+            SV* sv_value = Rstats::VectorFunc::get_value<Rstats::Character>(v1, i);
+            Rstats::VectorFunc::set_value<Rstats::Character>(v2, i, sv_value);
           }
           break;
         case Rstats::Type::COMPLEX :
           for (IV i = 0; i < length; i++) {
-            std::complex<NV> z = Rstats::VectorFunc::get_complex_value(v1, i);
+            std::complex<NV> z = Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i);
             NV re = z.real();
             NV im = z.imag();
             
@@ -541,12 +486,12 @@ namespace Rstats {
             sv_catpv(sv_str, SvPV_nolen(sv_im));
             sv_catpv(sv_str, "i");
 
-            Rstats::VectorFunc::set_character_value(v2, i, sv_str);
+            Rstats::VectorFunc::set_value<Rstats::Character>(v2, i, sv_str);
           }
           break;
         case Rstats::Type::DOUBLE :
           for (IV i = 0; i < length; i++) {
-            NV value = Rstats::VectorFunc::get_double_value(v1, i);
+            NV value = Rstats::VectorFunc::get_value<Rstats::Double>(v1, i);
             SV* sv_str = Rstats::pl_new_sv_pv("");
             if (std::isinf(value) && value > 0) {
               sv_catpv(sv_str, "Inf");
@@ -560,25 +505,25 @@ namespace Rstats {
             else {
               sv_catpv(sv_str, SvPV_nolen(Rstats::pl_new_sv_nv(value)));
             }
-            Rstats::VectorFunc::set_character_value(v2, i, sv_str);
+            Rstats::VectorFunc::set_value<Rstats::Character>(v2, i, sv_str);
           }
           break;
         case Rstats::Type::INTEGER :
           for (IV i = 0; i < length; i++) {
-            Rstats::VectorFunc::set_character_value(
+            Rstats::VectorFunc::set_value<Rstats::Character>(
               v2, 
               i,
-              Rstats::pl_new_sv_iv(Rstats::VectorFunc::get_integer_value(v1, i))
+              Rstats::pl_new_sv_iv(Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i))
             );
           }
           break;
         case Rstats::Type::LOGICAL :
           for (IV i = 0; i < length; i++) {
-            if (Rstats::VectorFunc::get_integer_value(v1, i)) {
-              Rstats::VectorFunc::set_character_value(v2, i, Rstats::pl_new_sv_pv("TRUE"));
+            if (Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i)) {
+              Rstats::VectorFunc::set_value<Rstats::Character>(v2, i, Rstats::pl_new_sv_pv("TRUE"));
             }
             else {
-              Rstats::VectorFunc::set_character_value(v2, i, Rstats::pl_new_sv_pv("FALSE"));
+              Rstats::VectorFunc::set_value<Rstats::Character>(v2, i, Rstats::pl_new_sv_pv("FALSE"));
             }
           }
           break;
@@ -599,11 +544,11 @@ namespace Rstats {
       switch (type) {
         case Rstats::Type::CHARACTER :
           for (IV i = 0; i < length; i++) {
-            SV* sv_value = Rstats::VectorFunc::get_character_value(v1, i);
+            SV* sv_value = Rstats::VectorFunc::get_value<Rstats::Character>(v1, i);
             SV* sv_value_fix = Rstats::Util::looks_like_double(sv_value);
             if (SvOK(sv_value_fix)) {
               NV value = SvNV(sv_value_fix);
-              Rstats::VectorFunc::set_double_value(v2, i, value);
+              Rstats::VectorFunc::set_value<Rstats::Double>(v2, i, value);
             }
             else {
               warn("NAs introduced by coercion");
@@ -614,18 +559,18 @@ namespace Rstats {
         case Rstats::Type::COMPLEX :
           warn("imaginary parts discarded in coercion");
           for (IV i = 0; i < length; i++) {
-            Rstats::VectorFunc::set_double_value(v2, i, Rstats::VectorFunc::get_complex_value(v1, i).real());
+            Rstats::VectorFunc::set_value<Rstats::Double>(v2, i, Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i).real());
           }
           break;
         case Rstats::Type::DOUBLE :
           for (IV i = 0; i < length; i++) {
-            Rstats::VectorFunc::set_double_value(v2, i, Rstats::VectorFunc::get_double_value(v1, i));
+            Rstats::VectorFunc::set_value<Rstats::Double>(v2, i, Rstats::VectorFunc::get_value<Rstats::Double>(v1, i));
           }
           break;
         case Rstats::Type::INTEGER :
         case Rstats::Type::LOGICAL :
           for (IV i = 0; i < length; i++) {
-            Rstats::VectorFunc::set_double_value(v2, i, Rstats::VectorFunc::get_integer_value(v1, i));
+            Rstats::VectorFunc::set_value<Rstats::Double>(v2, i, Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i));
           }
           break;
         default:
@@ -649,11 +594,11 @@ namespace Rstats {
       switch (type) {
         case Rstats::Type::CHARACTER :
           for (IV i = 0; i < length; i++) {
-            SV* sv_value = Rstats::VectorFunc::get_character_value(v1, i);
+            SV* sv_value = Rstats::VectorFunc::get_value<Rstats::Character>(v1, i);
             SV* sv_value_fix = Rstats::Util::looks_like_double(sv_value);
             if (SvOK(sv_value_fix)) {
               IV value = SvIV(sv_value_fix);
-              Rstats::VectorFunc::set_integer_value(v2, i, value);
+              Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, value);
             }
             else {
               warn("NAs introduced by coercion");
@@ -664,25 +609,25 @@ namespace Rstats {
         case Rstats::Type::COMPLEX :
           warn("imaginary parts discarded in coercion");
           for (IV i = 0; i < length; i++) {
-            Rstats::VectorFunc::set_integer_value(v2, i, (IV)Rstats::VectorFunc::get_complex_value(v1, i).real());
+            Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, (IV)Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i).real());
           }
           break;
         case Rstats::Type::DOUBLE :
           NV value;
           for (IV i = 0; i < length; i++) {
-            value = Rstats::VectorFunc::get_double_value(v1, i);
+            value = Rstats::VectorFunc::get_value<Rstats::Double>(v1, i);
             if (std::isnan(value) || std::isinf(value)) {
               Rstats::VectorFunc::add_na_position(v2, i);
             }
             else {
-              Rstats::VectorFunc::set_integer_value(v2, i, (IV)value);
+              Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, (IV)value);
             }
           }
           break;
         case Rstats::Type::INTEGER :
         case Rstats::Type::LOGICAL :
           for (IV i = 0; i < length; i++) {
-            Rstats::VectorFunc::set_integer_value(v2, i, Rstats::VectorFunc::get_integer_value(v1, i));
+            Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i));
           }
           break;
         default:
@@ -702,7 +647,7 @@ namespace Rstats {
       switch (type) {
         case Rstats::Type::CHARACTER :
           for (IV i = 0; i < length; i++) {
-            SV* sv_value = Rstats::VectorFunc::get_character_value(v1, i);
+            SV* sv_value = Rstats::VectorFunc::get_value<Rstats::Character>(v1, i);
             SV* sv_z = Rstats::Util::looks_like_complex(sv_value);
             
             if (SvOK(sv_z)) {
@@ -710,7 +655,7 @@ namespace Rstats {
               SV* sv_im = Rstats::pl_hv_fetch(sv_z, "im");
               NV re = SvNV(sv_re);
               NV im = SvNV(sv_im);
-              Rstats::VectorFunc::set_complex_value(v2, i, std::complex<NV>(re, im));
+              Rstats::VectorFunc::set_value<Rstats::Complex>(v2, i, std::complex<NV>(re, im));
             }
             else {
               warn("NAs introduced by coercion");
@@ -720,24 +665,24 @@ namespace Rstats {
           break;
         case Rstats::Type::COMPLEX :
           for (IV i = 0; i < length; i++) {
-            Rstats::VectorFunc::set_complex_value(v2, i, Rstats::VectorFunc::get_complex_value(v1, i));
+            Rstats::VectorFunc::set_value<Rstats::Complex>(v2, i, Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i));
           }
           break;
         case Rstats::Type::DOUBLE :
           for (IV i = 0; i < length; i++) {
-            NV value = Rstats::VectorFunc::get_double_value(v1, i);
+            NV value = Rstats::VectorFunc::get_value<Rstats::Double>(v1, i);
             if (std::isnan(value)) {
               Rstats::VectorFunc::add_na_position(v2, i);
             }
             else {
-              Rstats::VectorFunc::set_complex_value(v2, i, std::complex<NV>(Rstats::VectorFunc::get_double_value(v1, i), 0));
+              Rstats::VectorFunc::set_value<Rstats::Complex>(v2, i, std::complex<NV>(Rstats::VectorFunc::get_value<Rstats::Double>(v1, i), 0));
             }
           }
           break;
         case Rstats::Type::INTEGER :
         case Rstats::Type::LOGICAL :
           for (IV i = 0; i < length; i++) {
-            Rstats::VectorFunc::set_complex_value(v2, i, std::complex<NV>(Rstats::VectorFunc::get_integer_value(v1, i), 0));
+            Rstats::VectorFunc::set_value<Rstats::Complex>(v2, i, std::complex<NV>(Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i), 0));
           }
           break;
         default:
@@ -756,14 +701,14 @@ namespace Rstats {
       switch (type) {
         case Rstats::Type::CHARACTER :
           for (IV i = 0; i < length; i++) {
-            SV* sv_value = Rstats::VectorFunc::get_character_value(v1, i);
+            SV* sv_value = Rstats::VectorFunc::get_value<Rstats::Character>(v1, i);
             SV* sv_logical = Rstats::Util::looks_like_logical(sv_value);
             if (SvOK(sv_logical)) {
               if (SvTRUE(sv_logical)) {
-                Rstats::VectorFunc::set_integer_value(v2, i, 1);
+                Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, 1);
               }
               else {
-                Rstats::VectorFunc::set_integer_value(v2, i, 0);
+                Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, 0);
               }
             }
             else {
@@ -775,27 +720,27 @@ namespace Rstats {
         case Rstats::Type::COMPLEX :
           warn("imaginary parts discarded in coercion");
           for (IV i = 0; i < length; i++) {
-            Rstats::VectorFunc::set_integer_value(v2, i, Rstats::VectorFunc::get_complex_value(v1, i).real() ? 1 : 0);
+            Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i).real() ? 1 : 0);
           }
           break;
         case Rstats::Type::DOUBLE :
           for (IV i = 0; i < length; i++) {
-            NV value = Rstats::VectorFunc::get_double_value(v1, i);
+            NV value = Rstats::VectorFunc::get_value<Rstats::Double>(v1, i);
             if (std::isnan(value)) {
               Rstats::VectorFunc::add_na_position(v2, i);
             }
             else if (std::isinf(value)) {
-              Rstats::VectorFunc::set_integer_value(v2, i, 1);
+              Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, 1);
             }
             else {
-              Rstats::VectorFunc::set_integer_value(v2, i, value ? 1 : 0);
+              Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, value ? 1 : 0);
             }
           }
           break;
         case Rstats::Type::INTEGER :
         case Rstats::Type::LOGICAL :
           for (IV i = 0; i < length; i++) {
-            Rstats::VectorFunc::set_integer_value(v2, i, Rstats::VectorFunc::get_integer_value(v1, i) ? 1 : 0);
+            Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i) ? 1 : 0);
           }
           break;
         default:
@@ -807,7 +752,7 @@ namespace Rstats {
       return v2;
     }
 
-    SV* get_value(Rstats::Vector* v1, IV pos) {
+    SV* create_sv_value(Rstats::Vector* v1, IV pos) {
 
       SV* sv_value;
       
@@ -818,7 +763,7 @@ namespace Rstats {
             sv_value = &PL_sv_undef;
           }
           else {
-            sv_value = Rstats::VectorFunc::get_character_value(v1, pos);
+            sv_value = Rstats::VectorFunc::get_value<Rstats::Character>(v1, pos);
           }
           break;
         case Rstats::Type::COMPLEX :
@@ -826,7 +771,7 @@ namespace Rstats {
             sv_value = &PL_sv_undef;
           }
           else {
-            std::complex<NV> z = Rstats::VectorFunc::get_complex_value(v1, pos);
+            std::complex<NV> z = Rstats::VectorFunc::get_value<Rstats::Complex>(v1, pos);
             
             NV re = z.real();
             SV* sv_re;
@@ -868,7 +813,7 @@ namespace Rstats {
             sv_value = &PL_sv_undef;
           }
           else {
-            NV value = Rstats::VectorFunc::get_double_value(v1, pos);
+            NV value = Rstats::VectorFunc::get_value<Rstats::Double>(v1, pos);
             if (std::isnan(value)) {
               sv_value = Rstats::pl_new_sv_pv("NaN");
             }
@@ -889,7 +834,7 @@ namespace Rstats {
             sv_value = &PL_sv_undef;
           }
           else {
-            IV value = Rstats::VectorFunc::get_integer_value(v1, pos);
+            IV value = Rstats::VectorFunc::get_value<Rstats::Integer>(v1, pos);
             sv_value = Rstats::pl_new_sv_iv(value);
           }
           break;
@@ -912,8 +857,8 @@ namespace Rstats {
           v2 = Rstats::VectorFunc::new_complex(length);
           std::complex<NV> v2_total(1, 0);
           for (IV i = 0; i < length; i++) {
-            v2_total *= Rstats::VectorFunc::get_complex_value(v1, i);
-            Rstats::VectorFunc::set_complex_value(v2, i, v2_total);
+            v2_total *= Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i);
+            Rstats::VectorFunc::set_value<Rstats::Complex>(v2, i, v2_total);
           }
           break;
         }
@@ -921,8 +866,8 @@ namespace Rstats {
           v2 = Rstats::VectorFunc::new_double(length);
           NV v2_total(1);
           for (IV i = 0; i < length; i++) {
-            v2_total *= Rstats::VectorFunc::get_double_value(v1, i);
-            Rstats::VectorFunc::set_double_value(v2, i, v2_total);
+            v2_total *= Rstats::VectorFunc::get_value<Rstats::Double>(v1, i);
+            Rstats::VectorFunc::set_value<Rstats::Double>(v2, i, v2_total);
           }
           break;
         }
@@ -931,8 +876,8 @@ namespace Rstats {
           v2 = Rstats::VectorFunc::new_integer(length);
           IV v2_total(1);
           for (IV i = 0; i < length; i++) {
-            v2_total *= Rstats::VectorFunc::get_integer_value(v1, i);
-            Rstats::VectorFunc::set_integer_value(v2, i, v2_total);
+            v2_total *= Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i);
+            Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, v2_total);
           }
           break;
         }
@@ -958,8 +903,8 @@ namespace Rstats {
           v2 = Rstats::VectorFunc::new_complex(length);
           std::complex<NV> v2_total(0, 0);
           for (IV i = 0; i < length; i++) {
-            v2_total += Rstats::VectorFunc::get_complex_value(v1, i);
-            Rstats::VectorFunc::set_complex_value(v2, i, v2_total);
+            v2_total += Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i);
+            Rstats::VectorFunc::set_value<Rstats::Complex>(v2, i, v2_total);
           }
           break;
         }
@@ -967,8 +912,8 @@ namespace Rstats {
           v2 = Rstats::VectorFunc::new_double(length);
           NV v2_total(0);
           for (IV i = 0; i < length; i++) {
-            v2_total += Rstats::VectorFunc::get_double_value(v1, i);
-            Rstats::VectorFunc::set_double_value(v2, i, v2_total);
+            v2_total += Rstats::VectorFunc::get_value<Rstats::Double>(v1, i);
+            Rstats::VectorFunc::set_value<Rstats::Double>(v2, i, v2_total);
           }
           break;
         }
@@ -977,8 +922,8 @@ namespace Rstats {
           v2 = Rstats::VectorFunc::new_integer(length);
           IV v2_total(0);
           for (IV i = 0; i < length; i++) {
-            v2_total += Rstats::VectorFunc::get_integer_value(v1, i);
-            Rstats::VectorFunc::set_integer_value(v2, i, v2_total);
+            v2_total += Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i);
+            Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, v2_total);
           }
           break;
         }
@@ -1004,18 +949,18 @@ namespace Rstats {
           v2 = Rstats::VectorFunc::new_complex(1);
           std::complex<NV> v2_total(1, 0);
           for (IV i = 0; i < length; i++) {
-            v2_total *= Rstats::VectorFunc::get_complex_value(v1, i);
+            v2_total *= Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i);
           }
-          Rstats::VectorFunc::set_complex_value(v2, 0, v2_total);
+          Rstats::VectorFunc::set_value<Rstats::Complex>(v2, 0, v2_total);
           break;
         }
         case Rstats::Type::DOUBLE : {
           v2 = Rstats::VectorFunc::new_double(1);
           NV v2_total(1);
           for (IV i = 0; i < length; i++) {
-            v2_total *= Rstats::VectorFunc::get_double_value(v1, i);
+            v2_total *= Rstats::VectorFunc::get_value<Rstats::Double>(v1, i);
           }
-          Rstats::VectorFunc::set_double_value(v2, 0, v2_total);
+          Rstats::VectorFunc::set_value<Rstats::Double>(v2, 0, v2_total);
           break;
         }
         case Rstats::Type::INTEGER :
@@ -1023,9 +968,9 @@ namespace Rstats {
           v2 = Rstats::VectorFunc::new_integer(1);
           IV v2_total(1);
           for (IV i = 0; i < length; i++) {
-            v2_total *= Rstats::VectorFunc::get_integer_value(v1, i);
+            v2_total *= Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i);
           }
-          Rstats::VectorFunc::set_integer_value(v2, 0, v2_total);
+          Rstats::VectorFunc::set_value<Rstats::Integer>(v2, 0, v2_total);
           break;
         }
         default:
@@ -1056,18 +1001,18 @@ namespace Rstats {
           v2 = Rstats::VectorFunc::new_complex(1);
           std::complex<NV> v2_total(0, 0);
           for (IV i = 0; i < length; i++) {
-            v2_total += Rstats::VectorFunc::get_complex_value(v1, i);
+            v2_total += Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i);
           }
-          Rstats::VectorFunc::set_complex_value(v2, 0, v2_total);
+          Rstats::VectorFunc::set_value<Rstats::Complex>(v2, 0, v2_total);
           break;
         }
         case Rstats::Type::DOUBLE : {
           v2 = Rstats::VectorFunc::new_double(1);
           NV v2_total(0);
           for (IV i = 0; i < length; i++) {
-            v2_total += Rstats::VectorFunc::get_double_value(v1, i);
+            v2_total += Rstats::VectorFunc::get_value<Rstats::Double>(v1, i);
           }
-          Rstats::VectorFunc::set_double_value(v2, 0, v2_total);
+          Rstats::VectorFunc::set_value<Rstats::Double>(v2, 0, v2_total);
           break;
         }
         case Rstats::Type::INTEGER :
@@ -1075,9 +1020,9 @@ namespace Rstats {
           v2 = Rstats::VectorFunc::new_integer(1);
           IV v2_total(0);
           for (IV i = 0; i < length; i++) {
-            v2_total += Rstats::VectorFunc::get_integer_value(v1, i);
+            v2_total += Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i);
           }
-          Rstats::VectorFunc::set_integer_value(v2, 0, v2_total);
+          Rstats::VectorFunc::set_value<Rstats::Integer>(v2, 0, v2_total);
           break;
         }
         default:
@@ -1104,31 +1049,31 @@ namespace Rstats {
         case Rstats::Type::CHARACTER :
           v2 = Rstats::VectorFunc::new_character(length);
           for (IV i = 0; i < length; i++) {
-            Rstats::VectorFunc::set_character_value(v2, i, Rstats::VectorFunc::get_character_value(v1, i));
+            Rstats::VectorFunc::set_value<Rstats::Character>(v2, i, Rstats::VectorFunc::get_value<Rstats::Character>(v1, i));
           }
           break;
         case Rstats::Type::COMPLEX :
           v2 = Rstats::VectorFunc::new_complex(length);
           for (IV i = 0; i < length; i++) {
-            Rstats::VectorFunc::set_complex_value(v2, i, Rstats::VectorFunc::get_complex_value(v1, i));
+            Rstats::VectorFunc::set_value<Rstats::Complex>(v2, i, Rstats::VectorFunc::get_value<Rstats::Complex>(v1, i));
           }
           break;
         case Rstats::Type::DOUBLE :
           v2 = Rstats::VectorFunc::new_double(length);
           for (IV i = 0; i < length; i++) {
-            Rstats::VectorFunc::set_double_value(v2, i, Rstats::VectorFunc::get_double_value(v1, i));
+            Rstats::VectorFunc::set_value<Rstats::Double>(v2, i, Rstats::VectorFunc::get_value<Rstats::Double>(v1, i));
           }
           break;
         case Rstats::Type::INTEGER :
           v2 = Rstats::VectorFunc::new_integer(length);
           for (IV i = 0; i < length; i++) {
-            Rstats::VectorFunc::set_integer_value(v2, i, Rstats::VectorFunc::get_integer_value(v1, i));
+            Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i));
           }
           break;
         case Rstats::Type::LOGICAL :
           v2 = Rstats::VectorFunc::new_logical(length);
           for (IV i = 0; i < length; i++) {
-            Rstats::VectorFunc::set_integer_value(v2, i, Rstats::VectorFunc::get_integer_value(v1, i));
+            Rstats::VectorFunc::set_value<Rstats::Integer>(v2, i, Rstats::VectorFunc::get_value<Rstats::Integer>(v1, i));
           }
           break;
         default:
